@@ -85,11 +85,12 @@ class OvnProviderHelper(object):
     @staticmethod
     def _is_lb_empty(external_ids):
         """Check if there is no pool or listener defined."""
-        return not any([k.startswith('listener') or k.startswith('pool')
-                        for k in external_ids])
+        return not any(k.startswith('listener') or k.startswith('pool')
+                       for k in external_ids)
 
     @staticmethod
     def _delete_disabled_from_status(status):
+        # pylint: disable=multiple-statements
         d_regex = ':%s$' % ovn_const.DISABLED_RESOURCE_SUFFIX
         return {
             k: [{c: re.sub(d_regex, '', d) for c, d in i.items()}
@@ -115,6 +116,7 @@ class OvnProviderHelper(object):
         self.requests.put({'type': ovn_const.REQ_TYPE_EXIT})
         self.helper_thread.join()
         self.ovn_nbdb.stop()
+        del self.ovn_nbdb_api
 
     @staticmethod
     def _map_val(row, col, key):
@@ -122,9 +124,9 @@ class OvnProviderHelper(object):
         # and is expected to be caught by the caller.
         try:
             return getattr(row, col)[key]
-        except KeyError:
+        except KeyError as e:
             raise idlutils.RowNotFound(table=row._table.name,
-                                       col=col, match=key)
+                                       col=col, match=key) from e
 
     def _get_nw_router_info_on_interface_event(self, lrp):
         """Get the Router and Network information on an interface event
@@ -290,18 +292,24 @@ class OvnProviderHelper(object):
 
     def request_handler(self):
         while True:
-            try:
-                request = self.requests.get()
-                request_type = request['type']
-                if request_type == ovn_const.REQ_TYPE_EXIT:
-                    break
+            request = self.requests.get()
+            request_type = request['type']
+            if request_type == ovn_const.REQ_TYPE_EXIT:
+                break
 
-                request_handler = self._lb_request_func_maps.get(request_type)
+            request_handler = self._lb_request_func_maps.get(request_type)
+            try:
                 if request_handler:
                     status = request_handler(request['info'])
                     if status:
                         self._update_status_to_octavia(status)
                 self.requests.task_done()
+            except driver_exceptions.UpdateStatusError as e:
+                msg = ("Error while updating the load balancer "
+                       "status: %s") % e.fault_string
+                LOG.error(msg)
+                # TODO(haleyb): The resource(s) we were updating status for
+                # should be cleaned-up
             except Exception:
                 # If any unexpected exception happens we don't want the
                 # notify_loop to exit.
@@ -310,16 +318,16 @@ class OvnProviderHelper(object):
     def add_request(self, req):
         self.requests.put(req)
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(
+            driver_exceptions.UpdateStatusError),
+        wait=tenacity.wait_exponential(),
+        stop=tenacity.stop_after_delay(10),
+        reraise=True)
     def _update_status_to_octavia(self, status):
-        try:
-            status = OvnProviderHelper._delete_disabled_from_status(status)
-            LOG.debug('Updating status to octavia: %s', status)
-            self._octavia_driver_lib.update_loadbalancer_status(status)
-        except driver_exceptions.UpdateStatusError as e:
-            msg = ("Error while updating the load balancer "
-                   "status: %s") % e.fault_string
-            LOG.error(msg)
-            raise driver_exceptions.UpdateStatusError(msg)
+        status = OvnProviderHelper._delete_disabled_from_status(status)
+        LOG.debug('Updating status to octavia: %s', status)
+        self._octavia_driver_lib.update_loadbalancer_status(status)
 
     @tenacity.retry(
         retry=tenacity.retry_if_exception_type(idlutils.RowNotFound),
@@ -427,6 +435,10 @@ class OvnProviderHelper(object):
         lbs = self.ovn_nbdb_api.db_list_rows(
             'Load_Balancer').execute(check_error=True)
         for lb in lbs:
+            # Skip load balancers used by port forwarding plugin
+            if lb.external_ids.get(ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY) == (
+                    ovn_const.PORT_FORWARDING_PLUGIN):
+                continue
             if pool_key in lb.external_ids:
                 return lb
 
@@ -747,6 +759,7 @@ class OvnProviderHelper(object):
 
     @staticmethod
     def _get_selection_keys(lb_algorithm):
+        # pylint: disable=multiple-statements
         return ovn_const.LB_SELECTION_FIELDS_MAP[lb_algorithm]
 
     def check_lb_protocol(self, lb_id, listener_protocol):
@@ -776,7 +789,7 @@ class OvnProviderHelper(object):
                         break
 
         # If protocol set make sure its lowercase
-        protocol = protocol.lower() if protocol else None
+        protocol = protocol.lower() if protocol else []
         # In case port is not found for the vip_address we will see an
         # exception when port['id'] is accessed.
         external_ids = {
@@ -1244,8 +1257,12 @@ class OvnProviderHelper(object):
             self.ovn_nbdb_api.db_set(
                 'Load_Balancer', ovn_lb.uuid,
                 ('external_ids', external_ids)).execute(check_error=True)
-            # Pool status will be set to Online after a member is added to it.
+
+            # Pool status will be set to Online after a member is added to it
+            # or when it is created with listener.
             operating_status = constants.OFFLINE
+            if pool[constants.LISTENER_ID]:
+                operating_status = constants.ONLINE
 
             status = {
                 constants.POOLS: [
@@ -1466,11 +1483,13 @@ class OvnProviderHelper(object):
             pool = {constants.ID: member[constants.POOL_ID],
                     constants.PROVISIONING_STATUS: constants.ACTIVE,
                     constants.OPERATING_STATUS: constants.ONLINE}
+            member_status = {constants.ID: member[constants.ID],
+                             constants.PROVISIONING_STATUS: constants.ACTIVE}
+            if not member[constants.ADMIN_STATE_UP]:
+                member_status[constants.OPERATING_STATUS] = constants.OFFLINE
             status = {
                 constants.POOLS: [pool],
-                constants.MEMBERS: [
-                    {constants.ID: member[constants.ID],
-                     constants.PROVISIONING_STATUS: constants.ACTIVE}],
+                constants.MEMBERS: [member_status],
                 constants.LOADBALANCERS: [
                     {constants.ID: ovn_lb.name,
                      constants.PROVISIONING_STATUS: constants.ACTIVE}]}
@@ -1608,8 +1627,19 @@ class OvnProviderHelper(object):
             self._update_member(member, ovn_lb, pool_key)
             if constants.ADMIN_STATE_UP in member:
                 if member[constants.ADMIN_STATE_UP]:
-                    member_status[constants.OPERATING_STATUS] = (
-                        constants.ONLINE)
+                    old_admin_state_up = member.get('old_admin_state_up')
+                    if old_admin_state_up is None:
+                        exist_member = self._octavia_driver_lib.get_member(
+                            member[constants.ID])
+                        if exist_member:
+                            old_admin_state_up = exist_member.admin_state_up
+                    if old_admin_state_up:
+                        member_status[constants.OPERATING_STATUS] = (
+                            constants.ONLINE)
+                    else:
+                        # going from down to up should reflect NO_MONITOR state
+                        member_status[constants.OPERATING_STATUS] = (
+                            constants.NO_MONITOR)
                 else:
                     member_status[constants.OPERATING_STATUS] = (
                         constants.OFFLINE)
